//
// gpsr_sender.hh : GPSR Sender include file
// author         : Xi Wang
// note           : Converted from ping_sender
//
// Copyright (C) 2000-2003 by the University of Southern California
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#ifndef _GPSR_SENDER_HH_
#define _GPSR_SENDER_HH_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "diffapp.hh"

#include "gpsr_attrs.hh"

class GPSRSenderReceive;
class GPSRSenderApp;

#define SEND_DATA_INTERVAL 1

#ifdef NS_DIFFUSION
class GPSRSendDataTimer : public TimerHandler {
  public:
  GPSRSendDataTimer(GPSRSenderApp *a) : TimerHandler() { a_ = a; }
  void expire(Event *e);
protected:
  GPSRSenderApp *a_;
};
#endif //NS_DIFFUSION

class GPSRSenderApp : public DiffApp {
public:
#ifdef NS_DIFFUSION
  GPSRSenderApp();
  int command(int argc, const char*const* argv);
  void send();
#else
  GPSRSenderApp(int argc, char **argv);
  void parseCommandLine(int argc, char **argv);
#endif // NS_DIFFUSION

  virtual ~GPSRSenderApp()
  {
    // Nothing to do
  };

  void run();
  void recv(NRAttrVec *data, NR::handle my_handle);

  double dest_x, dest_y;
  enum modes {EXACT, NEAR} mode;
  int num_sends;
  enum algos {GPSR, GEAR, PUSH} algo;
  int trace_en;
  int my_id_;

private:
  // NR Specific variables
  GPSRSenderReceive *mr_;
  handle subHandle_;
  handle pubHandle_;

  // GPSR App variables
  int num_subscriptions_;
  int last_seq_sent_;
  NRAttrVec data_attr_;
  NRSimpleAttribute<void *> *timeAttr_;
  NRSimpleAttribute<int> *counterAttr_;
  GpsrEventTime *lastEventTime_;

  handle setupSubscription();
  handle setupPublication();
#ifdef NS_DIFFUSION
  GPSRSendDataTimer sdt_;
#endif // NS_DIFFUSION
};

class GPSRSenderReceive : public NR::Callback {
public:
  GPSRSenderReceive(GPSRSenderApp *app) : app_(app) {};
  void recv(NRAttrVec *data, NR::handle my_handle);

  GPSRSenderApp *app_;
};

#endif // !_GPSR_SENDER_HH_
