//
// rbp_receiver.cc : Rbp Receiver Main File
// author          : Fred Stann
//

#include "rbp_receiver.hh"

void RbpReceiverReceive::recv(NRAttrVec *data, NR::handle my_handle)
{
  app_->recv(data, my_handle);
}

void RbpReceiverApp::recv(NRAttrVec *data, NR::handle my_handle)
{
  NRSimpleAttribute<int> *counterAttr = NULL;
  NRSimpleAttribute<int> *sourceIDAttr = NULL;
  struct timeval tmv;
  int curMins;

  GetTime(&tmv);

  counterAttr = RbpCounterAttr.find(data);
  sourceIDAttr = RbpSrcIdAttr.find(data);

  if (!counterAttr || !sourceIDAttr){
    DiffPrint(DEBUG_ALWAYS, "RbpReceiver::Received a BAD packet !\n");
    PrintAttrs(data);
    return;
  }

  // Print output message
  curMins = (int)((tmv.tv_sec - startTime_.tv_sec)/60);
  DiffPrint(DEBUG_ALWAYS, "RbpReceiver::Received data %d from source %d at %d minutes!\n",
		counterAttr->getVal(), sourceIDAttr->getVal(), curMins);
}

handle RbpReceiverApp::setupSubscription()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::INTEREST_CLASS));
  attrs.push_back(NRAlgorithmAttr.make(NRAttribute::IS,
                                       NRAttribute::ONE_PHASE_PULL_ALGORITHM));
  // Use Two-Phase-Pull scoping for interest
  attrs.push_back(NRScopeAttr.make(NRAttribute::IS, NRAttribute::GLOBAL_SCOPE));
  attrs.push_back(RbpTargetAttr.make(NRAttribute::EQ, "Rbp"));

  DiffPrint(DEBUG_ALWAYS, "RbpReceiver::calling dr_->subscribe!\n");
  handle h = dr_->subscribe(&attrs, mr_);

  ClearAttrs(&attrs);

  return h;
}

void RbpReceiverApp::run()
{
  sleep(60);
  DiffPrint(DEBUG_ALWAYS, "RbpReceiverApp::run calls setupSubscription!\n");
  subHandle_ = setupSubscription();

  // Do nothing
  while (1){
    sleep(1000);
  }
}

RbpReceiverApp::RbpReceiverApp(int argc, char **argv)
{
  mr_ = new RbpReceiverReceive(this);

  parseCommandLine(argc, argv);
  dr_ = NR::createNR(diffusion_port_);
  GetTime(&startTime_);
}

int main(int argc, char **argv)
{
  RbpReceiverApp *app;

  sleep(5);
  app = new RbpReceiverApp(argc, argv);
  app->run();

  return 0;
}
